@@ -8,6 +8,9 @@
 #include "plasmacamera/path.h"
 #include "plasmacameramanager.h"
 
+#include <exiv2/exiv2.hpp>
+#include <memory>
+
 PlasmaCameraManager::PlasmaCameraManager(QObject *parent)
     : QObject(parent)
 {
@@ -16,6 +19,8 @@ PlasmaCameraManager::PlasmaCameraManager(QObject *parent)
     setQuality(static_cast<Quality>(CameraSettings::self()->videoRecordingQuality()));
     setVideoResolution(static_cast<Resolution>(CameraSettings::self()->videoResolution()));
     setVideoCodec(static_cast<VideoCodec>(CameraSettings::self()->videoCodec()));
+    setFilenamePattern(static_cast<FilenamePattern>(CameraSettings::self()->filenamePattern()));
+    setOutputPath(static_cast<OutputPath>(CameraSettings::self()->outputPath()));
 
     // Set to default audio input
     setAudioRecordingEnabledInternal(true);
@@ -61,6 +66,16 @@ PlasmaCameraManager::FileFormat PlasmaCameraManager::fileFormat() const
     return m_fileFormat;
 }
 
+PlasmaCameraManager::FilenamePattern PlasmaCameraManager::filenamePattern() const
+{
+    return m_filenamePattern;
+}
+
+PlasmaCameraManager::OutputPath PlasmaCameraManager::outputPath() const
+{
+    return m_outputPath;
+}
+
 QList<PlasmaCameraManager::FileFormat> PlasmaCameraManager::supportedFileFormats() const
 {
     return QList<FileFormat>({JPEG} );
@@ -158,6 +173,7 @@ void PlasmaCameraManager::startRecordingVideo()
     // Set the correct rotation metadata prior to recording
     QMediaMetaData metaData;
     metaData.insert(QMediaMetaData::Orientation, outputOrientationDegrees());
+    metaData.insert(QMediaMetaData::Date, QDateTime::currentDateTime());
     m_recorder->setMetaData(metaData);
 
     m_recorder->record();
@@ -204,6 +220,26 @@ void PlasmaCameraManager::setFileFormat(const FileFormat fileFormat)
     }
 }
 
+void PlasmaCameraManager::setFilenamePattern(const FilenamePattern filenamePattern)
+{
+    qDebug() << "PlasmaCameraManager::setFilenamePattern" << filenamePattern;
+    if (m_filenamePattern != filenamePattern) {
+        m_filenamePattern = filenamePattern;
+        CameraSettings::self()->setFilenamePattern(m_filenamePattern);
+        Q_EMIT filenamePatternChanged();
+    }
+}
+
+void PlasmaCameraManager::setOutputPath(const OutputPath outputPath)
+{
+    qDebug() << "PlasmaCameraManager::setOutputPath" << outputPath;
+    if (m_outputPath != outputPath) {
+        m_outputPath = outputPath;
+        CameraSettings::self()->setOutputPath(m_outputPath);
+        Q_EMIT outputPathChanged();
+    }
+}
+
 void PlasmaCameraManager::setQuality(const Quality quality)
 {
     if (m_quality != quality) {
@@ -503,8 +539,12 @@ void PlasmaCameraManager::processViewfinderFrame(const QImage &image)
 
 void PlasmaCameraManager::processCaptureImage(const QQueue<QImage> &frames)
 {
+    QDateTime now = QDateTime::currentDateTime();
+    if ( (m_filenamePattern==1) || (m_filenamePattern==2) ) {
+        m_imageCaptureFileName = now.toString(QStringLiteral("yyyyMMddHHmmss"));
+    }
     const QString fileName = PlasmaLibcameraUtils::generateFileName(
-        m_imageCaptureFileName, QStandardPaths::PicturesLocation, QLatin1String("jpg"));
+        m_imageCaptureFileName, QLatin1String("jpg"), m_filenamePattern, m_outputPath);
     QFile file(fileName);
 
     QImage image = frames.head();
@@ -531,12 +571,58 @@ void PlasmaCameraManager::processCaptureImage(const QQueue<QImage> &frames)
             QString::fromStdString("Could not save to file: %1").arg(fileName));
     }
 
-    // TODO: use QImageWriter to write EXIF data
-    //  - https://doc.qt.io/qt-6/qimagewriter.html
+    // Write EXIF data
+    writeExifData(fileName);
 
     Q_EMIT imageSaved(m_imageCaptureNum, fileName);
 }
 
+QString getExifOffsetFormat(int offsetSeconds)
+{
+    char sign = (offsetSeconds >= 0) ? '+' : '-';
+    int absOffsetSeconds = qAbs(offsetSeconds);
+    int hours = absOffsetSeconds / 3600;
+    int minutes = (absOffsetSeconds % 3600) / 60;
+
+    return QStringLiteral("%1%2:%3").arg(sign).arg(hours, 2, 10, QLatin1Char('0')).arg(minutes, 2, 10, QLatin1Char('0'));
+}
+
+void PlasmaCameraManager::writeExifData(const QString &fileName)
+{
+    try {
+        Exiv2::Image::UniquePtr image = Exiv2::ImageFactory::open(fileName.toStdString());
+        if (!image) {
+            return;
+        }
+
+        image->readMetadata();
+
+        Exiv2::ExifData &exifData = image->exifData();
+
+        // Write time and date related information
+        QDateTime now = QDateTime::currentDateTime();
+
+        std::string exifDateTime = now.toString(QStringLiteral("yyyy:MM:dd HH:mm:ss")).toStdString();
+        exifData["Exif.Image.DateTime"] = exifDateTime;
+        exifData["Exif.Photo.DateTimeOriginal"] = exifDateTime;
+        exifData["Exif.Photo.DateTimeDigitized"] = exifDateTime;
+
+        std::string exifOffset = getExifOffsetFormat(now.offsetFromUtc()).toStdString();
+        exifData["Exif.Photo.OffsetTime"] = exifOffset;
+        exifData["Exif.Photo.OffsetTimeOriginal"] = exifOffset;
+        exifData["Exif.Photo.OffsetTimeDigitized"] = exifOffset;
+
+        image->setExifData(exifData);
+        image->writeMetadata();
+    } catch (const Exiv2::Error &e) {
+        qWarning() << "Error writing EXIF data (exiv2 Error): " << e.what();
+        return;
+    } catch (const std::exception &e) {
+        qWarning() << "Error writing EXIF data: " << e.what();
+        return;
+    }
+}
+
 void PlasmaCameraManager::processVideoFrame()
 {
     // Set the time in the video that the frame should show up.
@@ -611,4 +697,4 @@ int PlasmaCameraManager::captureImageInternal()
 float PlasmaCameraManager::outputOrientationDegrees() const
 {
     return m_plasmaCamera->softwareRotationDegrees() + m_orientationSensorDegrees;
-}
\ No newline at end of file
+}
